/**
 * Created by jmoutard on 19/07/16.
 */

'use strict';
/*
 * Server Storage Service give data from the server if logged in and is capable of giving errors.
 * Errors are :
 *          - 0; Not authenticated (not implemented yet),
 *          - 1; Error when adding dashboard (There is no 'replace dashboard', this is the same error when adding a new dashboard or overriding an already existent one),
 *          - 2; Dashboard not found (same error for one and for all dashboard),
 *          - 3; Can't remove the dashboard.
 *
 * For convenience error code from 0 to 9 belongs to server storage errors.
 */
angular.module('adf.electre')
    .service('serverStorageService', [
        "$http", "$q", "parserService",
        function ($http, $q, parserService) {
            /**
             * @description Return all stored dashboard from the server.
             * @returns {Promise}
             */
            function getAll () {
                var deferred = $q.defer();
                $http.get('/lavoisier/list_layout')
                    .success(function(data){
                        /* Getting :
                         * <?xml-stylesheet type="text/xsl" href="/lavoisier/xsl/list_layout"?>
                         * <properties>
                         *     <dashboard id="[Dashboard id]">[Dashboard title]</dashboard>
                         *     ...
                         * </properties>
                         * [Dashboard id] and [Dashboard title] as String
                         */
                        /* First thing parse from String to XML */
                        var data_ = parserService.xmlDomFromString(data);
                        /* Second thing organise [Dashboard id] and [Dashboard title] like a JSON object in an Array */
                        var data_js = [], dashboards = data_.getElementsByTagName("dashboard");
                        for (var d=0 ; d<dashboards.length ; d++) {
                            data_js.push({"id":dashboards[d].id, "title":dashboards[d].innerHTML, state: "remote"});
                        }
                        deferred.resolve(data_js);
                    })
                    .error(function(){
                        deferred.reject();
                    });
                return deferred.promise;
            }

            /**
             * @description Return a specific stored dashboard from the server.
             * @param {string} id
             * @returns {Promise}
             */
            function get (id) {
                var deferred = $q.defer();
                $http.get('/lavoisier/load_layout?layout_id=' + id)
                    .success(function(data){
                        /* Getting :
                         * <dashboard>{...}</dashboard>
                         */
                        /* First thing parse from String to XML */
                        var data_ = parserService.xmlDomFromString(data);
                        /* Second thing navigate into XMLDOM to get the data */
                        data_ = data_.firstElementChild.firstChild.data;
                        /* At last give the data requested parsed into JSON */
                        deferred.resolve(JSON.parse(data_));
                    })
                    .error(function(){
                        deferred.reject();
                    });
                return deferred.promise;
            }

            /**
             * @description Add a dashboard onto the server.
             * @param {string} id
             * @param {object} data
             * @returns {Promise}
             */
            function add (id, data) {
                var deferred = $q.defer();
                $http.get('/lavoisier/save_layout?layout_id=' + id + "&data=" + JSON.stringify(data))
                    .success(function(data){
                        deferred.resolve();
                    })
                    .error(function(){
                        deferred.reject();
                    });
                return deferred.promise;
            }

            /**
             * @description Remove a dashboard onto the server.
             * @param {string} id
             * @returns {Promise}
             */
            function remove (id) {
                var deferred = $q.defer();
                $http.delete('/lavoisier/delete_layout?layout_id=' + id)
                    .success(function(){
                        deferred.resolve(true);
                    })
                    .error(function(){
                        deferred.reject();
                    });
                return deferred.promise;
            }

            return { getAll: getAll, get: get, add: add, remove: remove};
        }
    ]);